/*
 * AstriPacketGeneric.cpp
 *
 *  Created on: Jan 26, 2023
 *      Author: valerio
 */

#include <Astri_MA_Generic.h>
#include <Astri_MA_Recognized_Packet.h>
#include <ctime>
#include <iomanip>

using namespace inaf::oasbo::Packets;

uint AstriMaGeneric::getHeaderSize() const {
	return 6;
}

uint AstriMaGeneric::getPayloadSize() const {
	return getPacketLength() + 1;
}

uint AstriMaGeneric::getTailSize() const {
	return 0;
}

uint AstriMaGeneric::getTelescopeID() const {
	auto val = this->operator [](0);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.0 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getType() const {
	auto val = this->operator [](1);
	if (val.has_value())
		return val.value();
	else {
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[ASTRIMA Packet]\t"
				<< "Cannot access field n.1 to get Type." << std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getSubType() const {
	auto val = this->operator [](2);
	if (val.has_value())
		return val.value();
	else {
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[ASTRIMA Packet]\t"
				<< "cannot access field n.2 to get SubType." << std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getSSC() const {
	auto val = this->operator [](3);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.3 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getPacketLength() const {
	auto val = this->operator [](4);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.4 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getYear() const {
	auto val = this->operator [](5);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.5 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getMonth() const {
	auto val = this->operator [](6);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.6 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getDay() const {
	auto val = this->operator [](7);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.7 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getHour() const {
	auto val = this->operator [](8);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.8 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getMinutes() const {
	auto val = this->operator [](9);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.9 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getSeconds() const {
	auto val = this->operator [](10);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.10 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
bool AstriMaGeneric::getValidTime() const {
	auto val = this->operator [](11);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.11 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getTimeTagNanoseconds() const {
	auto val = this->operator [](12);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.12 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::getEventCounter() const {
	auto val = this->operator [](13);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.13 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
bool AstriMaGeneric::getLid() const {
	auto val = this->operator [](14);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.14 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
bool AstriMaGeneric::fibSt() const {
	auto val = this->operator [](15);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.15 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
bool AstriMaGeneric::fibCont() const {
	auto val = this->operator [](16);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.16 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
bool AstriMaGeneric::fibPuls() const {
	auto val = this->operator [](17);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.17 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::rgbCont() const {
	auto val = this->operator [](18);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.18 to get TelescopeID."	<< std::endl;
		return -1;
	}
}
uint AstriMaGeneric::rgbPuls() const {
	auto val = this->operator [](19);
	if (val.has_value())
		return val.value();
	else {
	    time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[AstriMA Packet]\t" << "Cannot access field n.19 to get TelescopeID."	<< std::endl;
		return -1;
	}
}

bool AstriMaGeneric::isRecognizedHeader() const {
	std::optional<size_t> type = this->operator []("Packet Type");
	if (!type.has_value())
		return false;
	std::optional<size_t> subtype = this->operator []("Packet SubType");
	if (!subtype.has_value())
		return false;
	std::optional<size_t> length = this->operator []("Packet Length");
	if (!length.has_value())
		return false;
	size_t totLength = this->getHeaderSize() + length.value() + 1
			+ this->getTailSize();

	std::tuple<size_t, size_t, size_t> target = std::make_tuple(type.value(),
			subtype.value(), totLength);
	auto it = std::find(AstriMa::recognizedPackets.begin(),
			AstriMa::recognizedPackets.end(), target);
	if (it != AstriMa::recognizedPackets.end())
		return true;
	else
		return false;
}

bool AstriMaGeneric::isRecognizedHeader(std::vector<uint8_t> buff) const {
	if (buff.size() != this->getHeaderSize())
		return false;

	size_t type = buff[1] >> 4;
	size_t subtype = buff[1] & 0x0F;
	ssize_t length = (buff[4] << 8) + buff[5];
	ssize_t totLength = this->getHeaderSize() + length + 1
			+ this->getTailSize();
	std::tuple<size_t, size_t, size_t> target = std::make_tuple(type, subtype,
			totLength);
	auto it = std::find(AstriMa::recognizedPackets.begin(),
			AstriMa::recognizedPackets.end(), target);
	if (it != AstriMa::recognizedPackets.end())
		return true;
	else
		return false;
}

